unit errmsgUtils;

{$mode delphi}{$H+}

interface

uses
  Classes, SysUtils, FileUtil;

type
  { TCompilerMsg }

  TCompilerMsg = class(TObject)
    MsgText : string;
    MsgType : string;
    Index : Integer;
    constructor Create(AMsgIndex: Integer; const AMsgType, AMsgText: string);
  end;

function ReadMessagesFromFile(const FileName: string; MsgList: TList): Boolean;
function ParseFileString(const s: AnsiString; var idx: Integer; var MsgType, MsgText: string): Boolean;

function GetDefaultMsgList(MsgList: TList): Boolean;
function GetCompMessage(index: Integer): TCompilerMsg;


var
  globalmsg : TList;

implementation

{$i msgidx.inc}
{$i msgtxt.inc}


type
  TMsgArray = array of TCompilerMsg;
var
  MsgArrayList : array [0..20] of TMsgArray;

function ParseDeclare(const s: AnsiString; i: Integer; var idx: Integer; var MsgType, MsgText: string): Boolean;
var
  j : Integer;
  t : string;
  err : integer;
begin
  j := i;
  t := Copy(s, j, 5);
  val(t, idx, err);
  Result := err = 0;
  if not Result then Exit;

  inc(j, 5);
  Result := (j < length(s)) and (s[j] = '_');
  if not Result then Exit;
  inc(j);

  i := j;
  while (j < length(s)) and (s[j] in ['A'..'Z', 'a'..'z']) do inc(j);
  MsgType := Copy(s, i, j - i);
  if j <= length(s) then begin
    Result := s[j] = '_';
    if not Result then Exit;
  end;
  inc(j);

  if j <= length(s) then begin
    i := j;
    while (j <= length(s)) and (s[j] <> #000) do inc(j);
    MsgText := Copy(s, i, j - i + 1)
  end else
    MsgText := '';

end;

function ParseFileString(const s: AnsiString; var idx: Integer; var MsgType, MsgText: string): Boolean;
var
  i : Integer;
begin
  i := Pos('=', s);
  Result := i > 0;
  if not Result then Exit;
  Result := ParseDeclare(s, i + 1, idx, MsgType, MsgText);
end;

function ReadMessagesFromFile(const FileName: string; MsgList: TList): Boolean;
var
  f : Text;
  s : string;
  idx : Integer;
  msg, txt: string;
begin
  if not FileExistsUTF8(FileName) then Exit;
  try
    system.Assign(f,FileName);
    try
      Reset(f);
      while not eof(f) do begin
        Readln(f, s);
        if ParseFileString(s, idx, msg, txt) then
          MsgList.Add(TCompilerMsg.Create(idx, msg, txt));
      end;
      Result := true;
    finally
      system.close(f)
    end;
  except
    Result := false;
  end;
end;

{ TCompilerMsg }

constructor TCompilerMsg.Create(AMsgIndex: Integer; const AMsgType,
  AMsgText: string);
begin
  Index := AMsgIndex;
  MsgType := AMsgType;
  MsgText := AMsgText;
end;

function GetDefaultMsgList(MsgList: TList): Boolean;
var
  s : AnsiString;
  i, idx : Integer;
  msg, txt : AnsiString;
begin
  Result := true;
  s := '';
  for i := 0 to length(msgtxt) - 1 do
    s := s + msgtxt[i];
  i := 1;
  repeat
    if ParseDeclare(s, i, idx, msg, txt) then
      MsgList.Add( TCompilerMsg.Create(idx, msg, txt));
    while (i <= length(s)) and (s[i] <> #000) do inc(i);
    while (i <= length(s)) and (s[i] = #000) do inc(i);
  until i > length(s)

end;



procedure InitCompMsg;
var
  i : Integer;
  idx : Integer;
  sdx : Integer;
  m   : TCompilerMsg;
begin
  globalmsg := TList.Create;
  GetDefaultMsgList(globalmsg);
  for i := 1 to 20 do
    SetLength(MsgArrayList[i], MsgIdxMax[i] + 1);
  for i := 0 to globalmsg.Count - 1 do begin
    m := TCompilerMsg(globalmsg[i]);
    idx := m.Index div 1000;
    sdx := m.Index mod 1000;
    MsgArrayList[idx][sdx] := m;
  end;
end;

function GetCompMessage(index: Integer): TCompilerMsg;
var
  idx, sdx : Integer;
begin
  Result := nil;
  idx := Index div 1000;
  sdx := Index mod 1000;
  if (idx < 0) or (idx >= length(MsgArrayList)) or (sdx >= length(MsgArrayList[idx])) then Exit;
  Result := MsgArrayList[idx][sdx];
end;

procedure FreeCompMsg;
var
  i : integer;
begin
  for i := 0 to globalmsg.Count - 1 do
     TObject(globalmsg[i]).Free;
  globalmsg.Free;
end;

initialization
  InitCompMsg;

finalization
  FreeCompMsg;

end.

