unit mainform;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, LResources, Forms, Controls, Graphics, Dialogs,
  ComCtrls, CheckLst, StdCtrls, Buttons, Menus, errmsgUtils;

type

  { TForm1 }

  TForm1 = class(TForm)
    CheckListBox1: TCheckListBox;
    SpeedButton1: TSpeedButton;
    SpeedButton2: TSpeedButton;
    SpeedButton3: TSpeedButton;
    SpeedButton4: TSpeedButton;
    Button1: TButton;
    Button2: TButton;
    MainMenu1: TMainMenu;
    MenuItem1: TMenuItem;
    MenuItem2: TMenuItem;
    MenuItem3: TMenuItem;
    MenuItem4: TMenuItem;
    MenuItem5: TMenuItem;
    MenuItem6: TMenuItem;
    OpenDialog1: TOpenDialog;
    SaveDialog1: TSaveDialog;
    procedure Button1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure MenuItem3Click(Sender: TObject);
    procedure MenuItem6Click(Sender: TObject);
    procedure CheckListBox1Click(Sender: TObject);
    procedure Memo1Change(Sender: TObject);
    procedure MenuItem2Click(Sender: TObject);
    procedure MenuItem4Click(Sender: TObject);
    procedure MenuItem5Click(Sender: TObject);
  private
    { private declarations }
  public
    { public declarations }
    confFileName  : string;
    modified      : Boolean;

    procedure SetConfFile(const FileName: string);
    procedure ResetAll;
    procedure OpenFromFile(const filename: string);
    procedure SaveToFile(const filename: string);
    procedure SaveAs;
    procedure AddCompMessage(MsgIndex: Integer; const MsgType, MsgText: string; AEnabled: Boolean = true);
    function GetValuesFromCheckLine(const s: string; var MsgIndex: Integer; var MsgType, MsgText: string): Boolean;
  end; 

var
  Form1: TForm1; 

implementation

function IntToStrLen(const int, len: Integer; FChar: char): string;
var
  s : string;
begin
  s := IntToStr(int);
  if length(s) < len then begin
    SetLength(Result, len);
    FillChar(Result[1], length(Result), FChar);
    if length(s) > 0 then
      Move(s[1], Result[ len - length(s)+1], length(s));
  end else
    Result := s;
end;


{ TForm1 }

procedure TForm1.Button1Click(Sender: TObject);
begin
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  CheckListBox1.Font.Name := 'Courier New';
  ResetAll;
end;

procedure TForm1.Button2Click(Sender: TObject);
begin
  ResetAll;
end;

procedure TForm1.Button4Click(Sender: TObject);
begin
  AddCompMessage(0, 'Hello world', 'E');
end;

procedure TForm1.Button3Click(Sender: TObject);
begin
end;

procedure TForm1.MenuItem3Click(Sender: TObject);
begin
end;

procedure TForm1.MenuItem6Click(Sender: TObject);
begin
  Close;
end;

procedure TForm1.CheckListBox1Click(Sender: TObject);
begin
end;

procedure TForm1.Memo1Change(Sender: TObject);
begin

end;

procedure TForm1.MenuItem2Click(Sender: TObject);
begin
  if OpenDialog1.Execute then begin
    SetConfFile(OpenDialog1.FileName);
    OpenFromFile(confFileName);
  end;
end;

procedure TForm1.MenuItem4Click(Sender: TObject);
begin
  SaveAs;
end;

procedure TForm1.MenuItem5Click(Sender: TObject);
begin
  if confFileName = '' then SaveAs
  else SaveToFile(confFileName);
end;

procedure TForm1.SetConfFile(const FileName: string);
begin
  if FileName = '' then Caption := 'FPC Messages Editor'
  else Caption := 'FPC Messages Editor: ' + FileName;
  confFileName := FileName;
end;

procedure TForm1.ResetAll;
var
  i : Integer;
begin
  CheckListBox1.BeginUpdateBounds;
  try
    CheckListBox1.Clear;
    for i := 0 to globalmsg.Count - 1 do
      with TCompilerMsg(globalmsg[i]) do begin
        if (length(MsgType) = 1) and (UpperCase(MsgType)[1] in ['W', 'N', 'H']) then begin
          AddCompMessage(Index, MsgType, MsgText);
        end;

      end;
  finally
    CheckListBox1.EndUpdateBounds;
  end;
end;

procedure TForm1.OpenFromFile(const filename: string);
var
  l : TList;
  b : array of Integer;
  i : integer;
  idx: integer;
  cm  : TCompilerMsg;
begin
  l := TList.Create;
  try
    if not ReadMessagesFromFile(filename, l) then Exit;

    SetLength(b, 15000);
    for i := 0 to length(b) do b[i] := -1;

    for i := 0 to l.Count - 1 do begin
      idx := TCompilerMsg(l[i]).Index;
      if (idx >= 0) and (idx < length(b)) then b[idx] := i;
    end;

    CheckListBox1.BeginUpdateBounds;
    try
      CheckListBox1.Clear;
      for i := 0 to globalmsg.Count - 1 do

        with TCompilerMsg(globalmsg[i]) do begin

          if (length(MsgType) = 1) and (UpperCase(MsgType)[1] in ['W', 'N', 'H']) then begin
            if (Index >= 0) and (Index < length(b)) and (b[Index] >= 0) then begin
              cm := TCompilerMsg( l[b[Index]]);
              if cm.MsgType = 'T' then AddCompMessage(Index, MsgType, MsgText, false)
              else AddCompMessage(Index, cm.MsgType, MsgText);
            end else
              AddCompMessage(Index, MsgType, MsgText);
          end;

        end;
    finally
      CheckListBox1.EndUpdateBounds;
    end;
  finally
    for i := 0 to l.Count - 1 do TObjecT(l[i]).Free;
    l.Free;
  end;
end;

procedure TForm1.SaveToFile(const filename: string);
var
  i : integer;
  idx : Integer;
  msg : string;
  txt : string;
  f   : System.Text;
begin
  if filename = '' then Exit;
  system.Assign(f, filename);
  try
    Rewrite(f);
    for i := 0 to CheckListBox1.Count - 1 do begin
      if GetValuesFromCheckLine(CheckListBox1.Items[i], idx, msg, txt) then begin
        if CheckListBox1.Checked[i]
          then writeln(f, '=' , IntToStrLen(idx, 5, '0'), '_'+msg+'_'+ txt)
          else writeln(f, '=', IntToStrLen(idx, 5, '0'), '_T_'+ txt);
      end;
    end;
  finally
    system.Close(f)
  end;
end;

procedure TForm1.SaveAs;
begin
  if SaveDialog1.Execute then begin
    SetConfFile(SaveDialog1.FileName);
    SaveToFile(confFileName)
  end;
end;


procedure TForm1.AddCompMessage(MsgIndex: Integer; const MsgType, MsgText: string; AEnabled : Boolean);
var
  i : Integer;
begin
  i := CheckListBox1.Items.Add(Format('(%s) %s %s', [MsgType, IntToStrLen(MsgIndex, 5, '0'), MsgText]));
  CheckListBox1.Checked[i] := AEnabled;
end;

function TForm1.GetValuesFromCheckLine(const s: string; var MsgIndex: Integer;
  var MsgType, MsgText: string): Boolean;
var
  err : integer;
begin
  MsgText := '';
  Result := false;
  if s = '' then Exit;
  MsgType := s[2];
  Val( Copy(s, 5, 5), MsgIndex, err);
  if err <> 0 then Exit;
  MsgText := Copy(s, 11, length(s) - 10);
  Result := true;
end;

initialization
  {$I mainform.lrs}

end.

